//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ListenSocket.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ListenSocket.h"
#include "opcodes.h"
#include "KnownFile.h"
#include "sharedfilelist.h"
#include "uploadqueue.h"
#include "uploadclient.h"

// CClientReqSocket
CClientReqSocket::CClientReqSocket(CPreferences* in_prefs,bool in_connected, CBaseClient* in_client){
	app_prefs = in_prefs;
	sizereceived = 0;
	sizetoget = 0;
	rbuffer = 0;
	headercomplete = false;
	pendingpacket = 0;
	pendingbuffer = 0;
	isbusy = false;
	client = in_client;
	connected = in_connected;
	disconnected = false;
}

CClientReqSocket::~CClientReqSocket(){
	safe_delete(pendingpacket);
	if (pendingbuffer)
		delete[] pendingbuffer;
}

void CClientReqSocket::OnClose(int nErrorCode){
	CAsyncSocket::OnClose(nErrorCode);
	Disconnect();
};

void CClientReqSocket::Disconnect(){
	connected = false;
	disconnected = true;
	Close();
	if (!client)
		delete this;
};

void CClientReqSocket::OnReceive(int nErrorCode){
	if  (!sizetoget || !headercomplete){
		if (!sizetoget){
			rbuffer = new char[5];
			headercomplete = false;
			sizereceived = this->Receive(rbuffer,5);
			if (sizereceived == SOCKET_ERROR){
				delete[] rbuffer;
				return;
			}
		}
		else
			sizereceived += this->Receive(rbuffer+sizereceived,sizetoget-sizereceived);

		if (sizereceived < 5){
			theApp.emuledlg->AddLogLine(false,"Error in Server:OnReceive! size < 5");
			sizetoget = 5;
			return;
		}
		else{
			Header_Struct* header = (Header_Struct*) rbuffer;
			if (header->eDonkeyID != OP_EDONKEYHEADER){
				theApp.emuledlg->AddLogLine(false,"Debug: Error in Server:OnReceive! header invalid");
				// errorcorrection: try to find next header somewhere		
				for (int i = 0;i != 5; i++){
					if (rbuffer[i] == OP_EDONKEYHEADER){
						memmove(rbuffer,rbuffer+i,5-i);
						sizetoget = 5;
						sizereceived = 5-i;
						return;
					}
				}
				delete[] rbuffer;
				sizereceived = 0;
				return;
			}
			else {
				headercomplete = true;
				sizetoget = header->packetlength;
				sizereceived = 0;
				delete[] rbuffer;
				return;
			}
		}
	}
	else {
		if (!rbuffer || !sizereceived)
			rbuffer = new char[sizetoget];
		sizereceived += this->Receive(rbuffer+sizereceived,sizetoget-sizereceived);
		if (sizetoget == sizereceived){
			if (!ProcessPacket(rbuffer+1, sizetoget-1, rbuffer[0]))
				return;
			delete[] rbuffer;
			sizetoget = 0;
			sizereceived = 0;
		}
	}
}

bool CClientReqSocket::ProcessPacket(char* packet, uint32 size, uint8 opcode){
	if (!client && opcode != OP_HELLO){
		Disconnect();
		return false;
	}
	switch(opcode){
		case OP_HELLO:{
			// create new upclient to save standart informations
			client = (CBaseClient*) new CUploadClient(packet,size,this);
			// send a response packet with standart informations
			CMemFile* data = new CMemFile();
			data->Write(app_prefs->GetUserHash(),16);
			uint32 clientid = theApp.serverconnect->GetClientID();
			data->Write(&clientid,4);
			uint16 port = app_prefs->GetPort();
			data->Write(&port,2);
			uint32 tagcount = 3;
			data->Write(&tagcount,4);
			CTag* temptag = new CTag(CT_NAME,app_prefs->GetUserNick());
			temptag->WriteTagToFile(data);
			delete temptag;
			temptag = new CTag(CT_VERSION,EDONKEYVERSION);
			temptag->WriteTagToFile(data);
			delete temptag;
			temptag = new CTag(CT_PORT,app_prefs->GetPort());
			temptag->WriteTagToFile(data);
			delete temptag;
			
			uint32 serverip = 0;
			uint16 serverport = 0;
			if (theApp.serverconnect->GetCurrentServer()){
				memcpy(&serverip,theApp.serverconnect->GetCurrentServer()->GetIP(),4);
				serverport = theApp.serverconnect->GetCurrentServer()->GetPort();
			}
			data->Write(&serverip,4);
			data->Write(&serverport,2);
			Packet* packet = new Packet(data);
			packet->opcode = OP_HELLOANSWER;
			delete data;
			SendPacket(packet,true);
			break;
		}
		case OP_FILEREQUEST:{
			if (size == 16){
				CKnownFile* reqfile = theApp.sharedfiles->GetFileByID((uchar*)packet);
				if (reqfile){
				// send filename etc
					memcpy(((CUploadClient*)client)->reqfileid,packet,16);
					CMemFile* data = new CMemFile();
					data->Write(reqfile->GetFileHash(),16);
					uint16 namelength = strlen(reqfile->GetFileName());
					data->Write(&namelength,2);
					data->Write(reqfile->GetFileName(),namelength);
					Packet* packet = new Packet(data);
					packet->opcode = OP_FILEREQANSWER;
					delete data;
					SendPacket(packet,true);
				//send filestatus
					data = new CMemFile();
					data->Write(reqfile->GetFileHash(),16);
					// TODO: find out how to submit status of incomplete files
					uint8 unknown = 0;
					data->Write(&unknown,1);
					data->Write(&unknown,1);
					data->Write(&unknown,1);
					packet = new Packet(data);
					packet->opcode = OP_FILESTATUS;
					delete data;
					SendPacket(packet,true);
					break;
				}
			}
			this->Disconnect();
			break;
		}
		case OP_STARTUPLOADREQ:{
			theApp.uploadqueue->AddClientToQueue((CUploadClient*)client);
			break;
		}
		case OP_REQUESTPARTS:{
			CMemFile* data = new CMemFile((BYTE*)packet,size);
			uchar reqfilehash[16];
			data->Read(reqfilehash,16);
			Requested_Block_Struct* reqblock1 = new Requested_Block_Struct;
			Requested_Block_Struct* reqblock2 = new Requested_Block_Struct;
			Requested_Block_Struct* reqblock3 = new Requested_Block_Struct;
			data->Read(&reqblock1->StartOffset,4);
			data->Read(&reqblock2->StartOffset,4);
			data->Read(&reqblock3->StartOffset,4);
			data->Read(&reqblock1->EndOffset,4);
			data->Read(&reqblock2->EndOffset,4);
			data->Read(&reqblock3->EndOffset,4);
			memcpy(&reqblock1->FileID,reqfilehash,16);
			memcpy(&reqblock2->FileID,reqfilehash,16);
			memcpy(&reqblock3->FileID,reqfilehash,16);
			if (reqblock1->EndOffset-reqblock1->StartOffset == 0)			
				delete reqblock1;
			else
				((CUploadClient*)client)->AddReqBlock(reqblock1);

			if (reqblock2->EndOffset-reqblock2->StartOffset == 0)			
				delete reqblock2;
			else
				((CUploadClient*)client)->AddReqBlock(reqblock2);

			if (reqblock3->EndOffset-reqblock3->StartOffset == 0)			
				delete reqblock3;
			else
				((CUploadClient*)client)->AddReqBlock(reqblock3);
			delete data;
			break;
		}
		case OP_CANCELTRANSFER:{
			Disconnect();
			return false;
		}
		case OP_HELLOANSWER:{
			uint32 test = client->GetState();
			if (client->GetState() == CS_UPLOADING){
				Packet* packet = new Packet(OP_ACCEPTUPLOADREQ,0);
				SendPacket(packet,true);
			}
			else
				Disconnect();
		}
		//default:
		  
	}
	return true;
}

bool CClientReqSocket::SendPacket(Packet* packet,bool delpacket){
	if (!IsConnected()){
		safe_delete(pendingpacket);
		if (!delpacket){
			Packet* copy = new Packet(packet->opcode,packet->size);
			memcpy(copy->pBuffer,packet->pBuffer,packet->size);
			pendingpacket = copy;
		}
		else
			pendingpacket = packet;
		return false;
	}
	if (Send(packet->GetHeader(),6))
		return false;
	if (packet->pBuffer){
		if (Send(packet->pBuffer,packet->size))
			return false;
	}
	if (delpacket)
		delete packet;
	return true;
}

void CClientReqSocket::OnConnect(int nErrorCode){
	CAsyncSocket::OnConnect(nErrorCode);
	if (nErrorCode){
		this->Disconnect();
	}
	else{
		connected = true;
		if (pendingpacket){
			this->SendPacket(pendingpacket,true);
			pendingpacket = 0;
		}
	}
}
void CClientReqSocket::OnSend(int nErrorCode){
	if (pendingbuffer){
		if (!Send(pendingbuffer,pendingblen,0,false)){
			delete[] pendingbuffer;
			pendingbuffer = 0;
		}
		else
			return;
	}
	CAsyncSocket::OnSend(nErrorCode);
	isbusy = false;
}

int CClientReqSocket::Send(const void*lpBuf,int nBufLen,int nFlags,bool safesend){
	if (CAsyncSocket::Send(lpBuf,nBufLen,nFlags) == SOCKET_ERROR){
		if (GetLastError() == WSAEWOULDBLOCK){
			isbusy = true;
			if (safesend){
				if (!pendingbuffer){
					pendingbuffer = new char[nBufLen];
					memcpy(pendingbuffer,lpBuf,nBufLen);
					pendingblen = nBufLen;
				}
				else{
					char* buffer = new char[pendingblen+nBufLen];
					memcpy(buffer,pendingbuffer,pendingblen);
					memcpy(&buffer[pendingblen],lpBuf,nBufLen);
					pendingblen += nBufLen;
					delete[] pendingbuffer;
					pendingbuffer = buffer;
				}
			}
			return -1;
		}
		else{
			Disconnect();
			return -1;
		}
	}
	return 0;
	
}

void CClientReqSocket::SendHelloPacket(){
	CMemFile* data = new CMemFile();
	uint8 hashsize = 16;
	data->Write(&hashsize,1);
	data->Write(app_prefs->GetUserHash(),16);
	uint32 clientid = theApp.serverconnect->GetClientID();
	data->Write(&clientid,4);
	uint16 port = app_prefs->GetPort();
	data->Write(&port,2);
	uint32 tagcount = 3;
	data->Write(&tagcount,4);
	CTag* tag = new CTag(CT_NAME,app_prefs->GetUserNick());
	tag->WriteTagToFile(data);
	delete tag;
	tag = new CTag(CT_VERSION,EDONKEYVERSION);
	tag->WriteTagToFile(data);
	delete tag;
	tag = new CTag(CT_PORT,app_prefs->GetPort());
	tag->WriteTagToFile(data);
	delete tag;
	char* ip;
	if (theApp.serverconnect->IsConnected()){
		ip = theApp.serverconnect->GetCurrentServer()->GetIP();
		port = theApp.serverconnect->GetCurrentServer()->GetPort();
	}
	else{
		port = 0;
		ip = "";
	}
	data->Write(ip,4);
	data->Write(&port,2);
	Packet* packet = new Packet(OP_HELLO,0);
	packet->size = data->GetLength();
	packet->pBuffer = (char*)data->Detach();
	delete data;
	SendPacket(packet,true);
}

// CListenSocket
// CListenSocket member functions
CListenSocket::CListenSocket(CPreferences* in_prefs){
	app_prefs = in_prefs;
}

CListenSocket::~CListenSocket(){
	Close();
}

bool CListenSocket::StartListening(){
	return (this->Create(app_prefs->GetPort()) && this->Listen());
}

void CListenSocket::OnAccept(int nErrorCode){
	//TODO: Block banned IPs here
	CClientReqSocket* newclient = new CClientReqSocket(app_prefs,true);
	this->Accept(*newclient);
}


