//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "UploadClient.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"
#include "uploadqueue.h"

CUploadClient::CUploadClient(char* hellodata,uint16 size,CClientReqSocket* sender)
	:CBaseClient() {
	SOCKADDR_IN sockAddr;
	memset(&sockAddr, 0, sizeof(sockAddr));
	uint32 nSockAddrLen = sizeof(sockAddr);
	sender->GetPeerName((SOCKADDR*)&sockAddr,(int*)&nSockAddrLen);
	userip = sockAddr.sin_addr.S_un.S_addr;
	strcpy(fulluserip,inet_ntoa(sockAddr.sin_addr));
	CMemFile* data = new CMemFile((BYTE*)hellodata,size);
	uint8 hashsize;
	data->Read(&hashsize,1);
	data->Read(&userhash,16);
	data->Read(&userid,4);
	data->Read(&userport,2); // hmm clientport is sent twice - why?
	uint32	tagcount;
	data->Read(&tagcount,4);
	for (int i = 0;i != tagcount; i++){
		CTag* temptag = new CTag(data);
		switch(temptag->tag->specialtag){
			case CT_NAME:
				username = strdup(temptag->tag->stringvalue);
				break;
			case CT_VERSION:
				clientversion = temptag->tag->intvalue;
				break;
			case CT_PORT:
				userport = temptag->tag->intvalue;
				break;
		}
		delete temptag;
	}
	data->Read(&serverip,4);
	data->Read(&serverport,2);
	delete data;
	starttime = ::GetTickCount();
	//TODO: complete this (friends, uploadspeed, emuleuser etc etc)
	isbotuser = strstr("finder.com",username);
	isfriend = false;
	isuploading = false;
	isemuleclient = false;
	filepriority = 10; // standart
	
	datarate = 0;
	upstarttime = 0;
	socket = sender;
	cur_block = 0;
}

uint32 CUploadClient::GetScore(){
	// calculate score, based on waitingtime and other factors
	float basevalue = (float)(::GetTickCount()-starttime)/1000;
	if (isbotuser)
		basevalue *= 0.6f;
	if (haslowid)
		basevalue *= 0.6f;
	if (isuploading)
		basevalue *= 2.5f;
	if (isfriend)
		basevalue *= 1.8f;
	if (isemuleclient)
		basevalue *= 1.3f;
	basevalue *= ((float)filepriority/10);
	return (uint32)basevalue;
}

CUploadClient::~CUploadClient(){
	if (cur_block){
		cur_block->Close();
		delete cur_block;
	}
	safe_delete(socket);
	for (POSITION pos = blockreqlist.GetHeadPosition();pos != 0;blockreqlist.GetNext(pos))
		delete blockreqlist.GetAt(pos);
	blockreqlist.RemoveAll();
	for (POSITION pos = doneblocklist.GetHeadPosition();pos != 0;doneblocklist.GetNext(pos))
		delete doneblocklist.GetAt(pos);
}

bool CUploadClient::CreateNextBlockPackage(){
	if (cur_block){
		cur_block->Close();
		delete cur_block;
		cur_block = 0;
	}
	if (blockreqlist.IsEmpty())
		return false;
	CMemFile* blockpack = new CMemFile();
	while (!blockreqlist.IsEmpty()){
		Requested_Block_Struct* currentblock = blockreqlist.GetHead();
		CKnownFile* srcfile = theApp.sharedfiles->GetFileByID(currentblock->FileID);
		if (!srcfile){
			socket->Disconnect();
			return false;
		}
		char* fullname = new char[strlen(srcfile->GetPath())+strlen(srcfile->GetFileName())+10];
		sprintf(fullname,"%s\\%s",srcfile->GetPath(),srcfile->GetFileName());
		FILE* file = fopen(fullname,"rbS");
		delete[] fullname;
		if (!file)
			continue;
		fseek(file,currentblock->StartOffset,0);
		uint32 togo;
		if (currentblock->StartOffset > currentblock->EndOffset){
			togo = currentblock->EndOffset + (srcfile->GetFileSize() - currentblock->StartOffset);
		}
		else
			togo = currentblock->EndOffset - currentblock->StartOffset;
		
		theApp.emuledlg->AddLogLine(false,"Debug: Pack From %i to %i",currentblock->StartOffset,currentblock->EndOffset);
		uint32 packetsize;
		if (togo > 10240) 
			packetsize = togo/(uint32)(togo/10240);
		else
			packetsize = togo;

		while (togo){
			if (togo < packetsize*2)
				packetsize = togo;
			togo -= packetsize;
			char* buffer = new char[packetsize+30];
			if (uint32 done = fread(&buffer[30],1,packetsize,file) != packetsize){
				fseek(file,0,0);
				fread(&buffer[30+done],packetsize-done,1,file);
			}
			if (ferror(file)){
				delete[] buffer;
				continue;
			}
			Packet* packet = new Packet(OP_SENDINGPART,0);
			packet->size = packetsize+24; // + 24 fileid + offsets (headersize is added at getheader)
			memcpy(buffer,packet->GetHeader(),6);
			delete packet;
			memcpy(&buffer[6],srcfile->GetFileHash(),16);
			uint32 statpos = (currentblock->EndOffset - togo) - packetsize;
			memcpy(&buffer[22],&statpos,4);
			uint32 endpos = (currentblock->EndOffset - togo);
			memcpy(&buffer[26],&endpos,4);
			blockpack->Write(buffer,packetsize+30);
			delete[] buffer;
		}
		fclose(file);
		memcpy(reqfileid,currentblock->FileID,16);
		doneblocklist.AddHead(blockreqlist.RemoveHead());
	}
	blockpack->SeekToBegin();
	cur_block = blockpack;
	theApp.emuledlg->AddLogLine(false,"Debug: Packet done. Size: %i",blockpack->GetLength());
	return true;
}

void CUploadClient::AddReqBlock(Requested_Block_Struct* reqblock){

	for (POSITION pos = doneblocklist.GetHeadPosition();pos != 0;doneblocklist.GetNext(pos)){
		if (reqblock->StartOffset == doneblocklist.GetAt(pos)->StartOffset && reqblock->EndOffset == doneblocklist.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	for (POSITION pos = blockreqlist.GetHeadPosition();pos != 0;blockreqlist.GetNext(pos)){
		if (reqblock->StartOffset == blockreqlist.GetAt(pos)->StartOffset && reqblock->EndOffset == blockreqlist.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	blockreqlist.AddTail(reqblock);

}

void CUploadClient::SetUpStartTime(uint32 time){
	if (time)
		upstarttime = time;
	else
		upstarttime = ::GetTickCount();
}

void CUploadClient::SetWaitStartTime(uint32 time){
	if (time)
		starttime = time;
	else
		starttime = ::GetTickCount();
}

uint32 CUploadClient::SendBlockData(uint32 maxammount){
	dataratetmp += dataratems;
	count++;
	if (count == 100){
		datarate = dataratetmp/10;
		count = 0;
		dataratetmp = 0;
		theApp.emuledlg->transferwnd.uploadlistctrl.RefreshClient(this);
	}
	dataratems = 0;
	if (!socket->IsBusy()){
		if (cur_block){
			char* sendbuffer = new char[maxammount];
			uint32 rdone = 0;
			rdone = cur_block->Read(sendbuffer,maxammount);
			socket->Send(sendbuffer,rdone);
			if (maxammount-rdone)
				CreateNextBlockPackage();
			delete[] sendbuffer;
			transfered += rdone;
			dataratems += rdone;
			return rdone;
		}
		else{
			CreateNextBlockPackage();
			return 0;
		}
	}
	return 0;
}