//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "server.h"
#include "opcodes.h"
#include "emule.h"


CServer::CServer(ServerMet_Struct* in_data){
	taglist = new CTypedPtrList<CPtrList, Tag_Struct*>;
	port = in_data->port;
	tagcount = 0;
	sprintf(ipfull,"%i.%i.%i.%i",in_data->ip[0],in_data->ip[1],in_data->ip[2],in_data->ip[3]);
	memcpy(ip,in_data->ip,4);
	files = 0;
	users = 0;
	preferences = 0;
	ping = 0;
	description = 0;
	name = 0;
	listname = 0;
}
CServer::CServer(uint16 in_port, int8 in_ip[4]){
	taglist = new CTypedPtrList<CPtrList, Tag_Struct*>;
	port = in_port;
	tagcount = 0;
	sprintf(ipfull,"%i.%i.%i.%i",in_ip[0],in_ip[1],in_ip[2],in_ip[3]);
	memcpy(ip,in_ip,4);
	files = 0;
	users = 0;
	preferences = 0;
	ping = 0;
	description = 0;
	name = 0;
	listname = 0;
}


CServer::~CServer(){
	safe_delete(description);
	safe_delete(name);
	safe_delete(listname);
	for(POSITION pos = taglist->GetHeadPosition(); pos != NULL; pos = taglist->GetHeadPosition()) {
		safe_delete(taglist->GetAt(pos)->tagname);
		safe_delete(taglist->GetAt(pos)->stringvalue);
		delete taglist->GetAt(pos);
		taglist->RemoveAt(pos);
	}
	delete taglist;
}

bool CServer::AddTagFromFile(FILE* servermet){
	if (servermet == 0)
		return false;
	int8 tagtype = fgetc(servermet);
	int16 taglen;
	fread(&taglen,1,2,servermet);
	char* buffer = new char[taglen+1];
	fread(buffer,taglen,1,servermet);
	buffer[taglen] = 0;
	int16 stringlen;
	if (taglen == 1){ // specialtag
		switch(buffer[0]){		
					case ST_SERVERNAME:
						fread(&stringlen,2,1,servermet);
						name = new char[stringlen+1];
						fread(name,stringlen,1, servermet);
						name[stringlen] = 0;
						break;
					case ST_DESCRIPTION:
						fread(&stringlen,1,2,servermet);
						description = new char[stringlen+1];
						fread(description,stringlen,1, servermet);
						description[stringlen] = 0;
						break;
					case ST_PREFERENCE:
						fread(&preferences,4,1, servermet);
						break;
					case ST_PING:
						fread(&ping,4,1, servermet);
						break;
					default:
						Tag_Struct* tag = new Tag_Struct;
						memset(tag,0,sizeof(Tag_Struct));
						tag->type = tagtype;
						tag->specialtag = buffer[0];
						tag->tagname = new char[8];
						memcpy(tag->tagname,"Unknown",7);
						tag->tagname[7] = 0;
						if (tag->type == 3)
							fread(&tag->intvalue,4,1,servermet);
						else if (tag->type == 2) {					
							fread(&stringlen,2,1,servermet);
							tag->stringvalue = new char[stringlen+1];
							fread(tag->stringvalue,stringlen,1, servermet);
							tag->stringvalue[stringlen] = 0;
						}
						AddTag(tag);
		}
	}
	else {
		// semi special tags
		if (!strcmp(buffer,"files") && tagtype == 3)
			fread(&files,4,1,servermet);
		else if (!strcmp(buffer,"users") && tagtype == 3)
			fread(&users,4,1,servermet);
		else {
			//standarttag
			Tag_Struct* tag = new Tag_Struct;
			memset(tag,0,sizeof(Tag_Struct));
			tag->type = tagtype;
			tag->tagname = new char[taglen+1];
			strcpy(tag->tagname,buffer);
			if (tag->type == 3)
				fread(&tag->intvalue,1,4,servermet);
			else if (tag->type == 2) {
				fread(&stringlen,1,2,servermet);
				tag->stringvalue = new char[stringlen+1];
				fread(tag->stringvalue,stringlen,1, servermet);
				tag->stringvalue[stringlen] = 0;
			}
			else {
				// TODO use error class
				theApp.emuledlg->AddLogLine(false,"Error while processing servertags: invalid tagtype");
			}
			AddTag(tag);
		}
	}
	delete [] buffer;
	return !ferror(servermet);
}

void CServer::SetListName(char* newname){
	if (listname)
		delete[] listname;
	listname = newname;
}

void CServer::FillWindowTags(CTreeCtrl* wnd,HTREEITEM rootitem){
	POSITION pos;
	char buffer[255];
	if (description) {
		sprintf(buffer,"Description: %s",description);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	sprintf(buffer,"IP: %s",ipfull);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	sprintf(buffer,"Port: %i",port);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	if (ping) {
		sprintf(buffer,"Ping: %i",ping);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (users) {
		sprintf(buffer,"Users: %i",users);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (files) {
		sprintf(buffer,"Files: %i",files);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	Tag_Struct* cur_tag;
	for(int i = 0; i != taglist->GetCount();i++) {
		pos = taglist->FindIndex(i);
		cur_tag = taglist->GetAt(pos);
		if (cur_tag->specialtag)
			continue;
		if (cur_tag->type == 2)
			sprintf(buffer,"%s: %s",cur_tag->tagname,cur_tag->stringvalue);
		else if (cur_tag->type == 3)
			sprintf(buffer,"%s: %i",cur_tag->tagname,cur_tag->intvalue);
		else
			continue;
		wnd->InsertItem(buffer,-1,-1,rootitem);
   }
}

void CServer::AddTag(CTag* in_tag){
	Tag_Struct* newtag = new Tag_Struct;
	memcpy(newtag,in_tag->tag,sizeof(Tag_Struct));
	taglist->AddTail(newtag);
}


	
	
